@@ -7,6 +7,7 @@ from __future__ import print_function, absolute_import
 
 import string
 
+from .. import six
 from ..six import StringIO
 from . import types, _utils
 from ._utils import _StrCaching, _StringReferenceCaching, _HasMetadata
@@ -14,13 +15,15 @@ from ._utils import _StrCaching, _StringReferenceCaching, _HasMetadata
 
 _VALID_CHARS = (frozenset(map(ord, string.ascii_letters)) |
                 frozenset(map(ord, string.digits)) |
-                frozenset('._-$'))
+                frozenset(map(ord, ' !#$%&\'()*+,-./:;<=>?@[]^_`{|}~')))
 
 
 def _escape_string(text, _map={}):
     """
     Escape the given bytestring for safe use as a LLVM array constant.
     """
+    if isinstance(text, str):
+        text = text.encode('ascii')
     assert isinstance(text, (bytes, bytearray))
 
     if not _map:
@@ -29,6 +32,8 @@ def _escape_string(text, _map={}):
                 _map[ch] = chr(ch)
             else:
                 _map[ch] = '\\%02x' % ch
+            if six.PY2:
+                _map[chr(ch)] = _map[ch]
 
     buf = [_map[ch] for ch in text]
     return ''.join(buf)
@@ -251,14 +256,16 @@ class MetaDataString(NamedValue):
     """
 
     def __init__(self, parent, string):
-        super(MetaDataString, self).__init__(parent, types.MetaData(), name="")
+        super(MetaDataString, self).__init__(parent,
+                                             types.MetaDataType(),
+                                             name="")
         self.string = string
 
     def descr(self, buf):
         buf += (self.get_reference(), "\n")
 
     def _get_reference(self):
-        return '!"{0}"'.format(self.string)
+        return '!"{0}"'.format(_escape_string(self.string))
 
     _to_string = _get_reference
 
@@ -275,7 +282,35 @@ class MetaDataString(NamedValue):
         return hash(self.string)
 
 
+class MetaDataArgument(_StrCaching, _StringReferenceCaching, Value):
+    """
+    An argument value to a function taking metadata arguments.
+    This can wrap any other kind of LLVM value.
+
+    Do not instantiate directly, Builder.call() will create these
+    automatically.
+    """
+
+    def __init__(self, value):
+        assert isinstance(value, Value)
+        assert not isinstance(value.type, types.MetaDataType)
+        self.type = types.MetaDataType()
+        self.wrapped_value = value
+
+    def _get_reference(self):
+        # e.g. "i32* %2"
+        return "{0} {1}".format(self.wrapped_value.type,
+                                self.wrapped_value.get_reference())
+
+    _to_string = _get_reference
+
+
 class NamedMetaData(object):
+    """
+    A named metadata node.
+
+    Do not instantiate directly, use Module.add_named_metadata() instead.
+    """
 
     def __init__(self, parent):
         self.parent = parent
@@ -286,22 +321,24 @@ class NamedMetaData(object):
 
 
 class MDValue(NamedValue):
+    """
+    A metadata node's value, consisting of a sequence of elements ("operands").
+
+    Do not instantiate directly, use Module.add_metadata() instead.
+    """
     name_prefix = '!'
 
     def __init__(self, parent, values, name):
-        super(MDValue, self).__init__(parent, types.MetaData(), name=name)
+        super(MDValue, self).__init__(parent,
+                                      types.MetaDataType(),
+                                      name=name)
         self.operands = tuple(values)
         parent.metadata.append(self)
 
-    @property
-    def operand_count(self):
-        return len(self.operands)
-
     def descr(self, buf):
         operands = []
         for op in self.operands:
-            typestr = str(op.type)
-            if isinstance(op.type, types.MetaData):
+            if isinstance(op.type, types.MetaDataType):
                 if isinstance(op, Constant) and op.constant == None:
                     operands.append("null")
                 else:
@@ -327,6 +364,78 @@ class MDValue(NamedValue):
         return hash(self.operands)
 
 
+class DIToken:
+    """
+    A debug information enumeration value that should appear bare in
+    the emitted metadata.
+
+    Use this to wrap known constants, e.g. the DW_* enumerations.
+    """
+    def __init__(self, value):
+        self.value = value
+
+
+class DIValue(NamedValue):
+    """
+    A debug information descriptor, containing key-value pairs.
+
+    Do not instantiate directly, use Module.add_debug_info() instead.
+    """
+    name_prefix = '!'
+
+    def __init__(self, parent, is_distinct, kind, operands, name):
+        super(DIValue, self).__init__(parent,
+                                      types.MetaDataType(),
+                                      name=name)
+        self.is_distinct = is_distinct
+        self.kind = kind
+        self.operands = tuple(operands)
+        parent.metadata.append(self)
+
+    def descr(self, buf):
+        if self.is_distinct:
+            buf += ("distinct ",)
+        operands = []
+        for key, value in self.operands:
+            if value is None:
+                strvalue = "null"
+            elif value is True:
+                strvalue = "true"
+            elif value is False:
+                strvalue = "false"
+            elif isinstance(value, DIToken):
+                strvalue = value.value
+            elif isinstance(value, str):
+                strvalue = '"{}"'.format(_escape_string(value))
+            elif isinstance(value, int):
+                strvalue = str(value)
+            elif isinstance(value, NamedValue):
+                strvalue = value.get_reference()
+            else:
+                raise TypeError("invalid operand type for debug info: %r"
+                                % (value,))
+            operands.append("{0}: {1}".format(key, strvalue))
+        operands = ', '.join(operands)
+        buf += ("!", self.kind, "(", operands, ")\n")
+
+    def _get_reference(self):
+        return self.name_prefix + str(self.name)
+
+    def __eq__(self, other):
+        if isinstance(other, DIValue):
+            return self.is_distinct == other.is_distinct and \
+                self.kind == other.kind and \
+                self.operands == other.operands
+        else:
+            return False
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __hash__(self):
+        return hash((self.is_distinct, self.kind, self.operands))
+
+
 class GlobalValue(NamedValue, _ConstOpMixin):
     """
     A global value.
@@ -349,11 +458,12 @@ class GlobalVariable(GlobalValue):
         assert isinstance(typ, types.Type)
         super(GlobalVariable, self).__init__(module, typ.as_pointer(addrspace),
                                              name=name)
-        self.gtype = typ
+        self.value_type = typ
         self.initializer = None
         self.unnamed_addr = False
         self.global_constant = False
         self.addrspace = addrspace
+        self.align = None
         self.parent.add_global(self)
 
     def descr(self, buf):
@@ -368,29 +478,30 @@ class GlobalVariable(GlobalValue):
         else:
             linkage = self.linkage
 
-        if self.addrspace != 0:
-            addrspace = 'addrspace({0:d})'.format(self.addrspace)
-        else:
-            addrspace = ''
-
+        if linkage:
+            buf.append(linkage + " ")
+        if self.storage_class:
+            buf.append(self.storage_class + " ")
         if self.unnamed_addr:
-            unnamed_addr = 'unnamed_addr'
-        else:
-            unnamed_addr = ''
+            buf.append("unnamed_addr ")
+        if self.addrspace != 0:
+            buf.append('addrspace({0:d}) '.format(self.addrspace))
 
-        buf.append("{linkage} {storage_class} {unnamed_addr} {addrspace} {kind} {type} "
-                   .format(linkage=linkage,
-                           storage_class=self.storage_class,
-                           unnamed_addr=unnamed_addr,
-                           addrspace=addrspace,
-                           kind=kind,
-                           type=self.gtype))
+        buf.append("{kind} {type}" .format(kind=kind, type=self.value_type))
 
         if self.initializer is not None:
-            buf.append(self.initializer.get_reference())
+            if self.initializer.type != self.value_type:
+                raise TypeError("got initializer of type %s "
+                                "for global value type %s"
+                                % (self.initializer.type, self.value_type))
+            buf.append(" " + self.initializer.get_reference())
         elif linkage not in ('external', 'extern_weak'):
             # emit 'undef' for non-external linkage GV
-            buf.append(Constant(self.gtype, Undefined).get_reference())
+            buf.append(" " + self.value_type(Undefined).get_reference())
+
+        if self.align is not None:
+            buf.append(", align %d" % (self.align,))
+
         buf.append("\n")
 
 
@@ -403,14 +514,15 @@ class AttributeSet(set):
 
 
 class FunctionAttributes(AttributeSet):
-    _known = frozenset(['alwaysinline', 'builtin', 'cold', 'inlinehint',
-                        'jumptable', 'minsize', 'naked', 'nobuiltin',
-                        'noduplicate', 'noimplicitfloat', 'noinline',
-                        'nonlazybind', 'noredzone', 'noreturn', 'nounwind',
-                        'optnone', 'optsize', 'readnone', 'readonly',
-                        'returns_twice', 'sanitize_address',
-                        'sanitize_memory', 'sanitize_thread', 'ssp',
-                        'sspreg', 'sspstrong', 'uwtable'])
+    _known = frozenset([
+        'argmemonly', 'alwaysinline', 'builtin', 'cold',
+        'inaccessiblememonly', 'inaccessiblemem_or_argmemonly', 'inlinehint',
+        'jumptable', 'minsize', 'naked', 'nobuiltin', 'noduplicate',
+        'noimplicitfloat', 'noinline', 'nonlazybind', 'norecurse',
+        'noredzone', 'noreturn', 'nounwind', 'optnone', 'optsize',
+        'readnone', 'readonly', 'returns_twice', 'sanitize_address',
+        'sanitize_memory', 'sanitize_thread', 'ssp',
+        'sspreg', 'sspstrong', 'uwtable'])
 
     def __init__(self):
         self._alignstack = 0
@@ -601,6 +713,10 @@ class Block(NamedValue):
     def function(self):
         return self.parent
 
+    @property
+    def module(self):
+        return self.parent.module
+
     def descr(self, buf):
         buf.append("{0}:\n".format(self.name))
         buf += ["  {0}\n".format(instr) for instr in self.instructions]
