@@ -269,7 +269,7 @@ class TestMisc(BaseTest):
 
     def test_version(self):
         major, minor, patch = llvm.llvm_version_info
-        self.assertIn((major, minor), [(3, 6), (3, 7)])
+        self.assertEqual((major, minor), (3, 8))
         self.assertIn(patch, range(10))
 
     def test_check_jit_execution(self):
